using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using MikuMikuDance.XNA;
using MikuMikuDance.XNA.Model;
using MikuMikuDance.XNA.Motion;

namespace MikuMikuDanceXNADemo2
{
    /// <summary>
    /// MikuMikuDance for XNAのデモ2
    /// このデモでは
    /// -ゲーム中でのモーションのベイク
    /// -コンテンツパイプライン中でのモーションのベイク
    /// が書かれています
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        //MikuMikuDance for XNA
        MikuMikuDanceXNA mmd;
        //MMDモデルデータ
        MMDModel model1,model2;
        //MMDモーションデータ
        MMDMotion motion1;
        //ベイク済みモーションデータ
        MMDBakedMotion motion2;

        public Game1()
        {
            //デバイスがMMDXの必要条件を満たしているかチェック
            graphics = new GraphicsDeviceManager(this);
            MikuMikuDanceXNA.CheckDevice(graphics); 
            Content.RootDirectory = "Content";
        }

        protected override void Initialize()
        {
            base.Initialize();
        }

        /// <summary>
        /// LoadContent関数
        /// 各種ロード処理はここでロードを行っておくと良い
        /// </summary>
        protected override void LoadContent()
        {
            //MikuMikuDanceXNAを作成する
            mmd = new MikuMikuDanceXNA(this);
            //プロジェクトのContent内に追加してあるMiku.pmd, Miku-metal.pmdモデルをロードする
            //ロードはMikuMikuDanceXNA.LoadModel関数で行う
            //(モデルデータはあにまさ氏よりお借りしました。このモデルファイルはMITライセンス適応外です)
            model1 = mmd.LoadModel("Miku", this, Matrix.CreateTranslation(-10, 0, -10));
            model2 = mmd.LoadModel("Miku-metal", this, Matrix.CreateTranslation(10, 0, -10));
            //同じくContent内に追加してあるTrueMyHeart.vmdモーションをロードする
            //ロードはMikuMikuDanceXNA.LoadMotion関数で行う
            motion1 = mmd.LoadMotion("TrueMyHeart");
            
            //model2はGPU頂点アニメーションモードをオンにする
            //GPU頂点アニメーションモードとは表情アニメーション計算をCPUではなくGPUで処理するモード
            //GPUがフルに使える環境(XBox360やデスクトップPC)なら高速化。ただし、GPUが弱いノートでは逆に遅い
            //Windwosはデフォルトでfalse,XBoxはデフォルトでtrue。コンパイルオプションのGPUAnimeで切り替え可能
            model2.GPUAnimation = true;

            //model1はモーションセット後、ゲーム中のベイクを行う
            //model2はコンテンツパイプライン中のベイクモーションをセットする
            //ベイクとは、MMDのボーン補間計算やIKボーンの計算といった重たい処理を
            //あらかじめ計算しておくことでCPUの負荷を減らし実行速度を大幅に上昇させる処理である
            //(XBox360のCPUは非常に弱いため、この処理は不可欠)
            //注意点
            //1.ベイク済みモーションは他の種類のモデルでの動作は保証しない(同じボーンなら動くかな？)
            //2.ベイクはメモリを消費する(動作に含まれるボーンや頂点数にもよるが、2分のモーションで1.2Mbyte〜)
            //3.ゲーム中のベイクをする場合、ベイク処理は当然ながら重たい(マルチスレッド化して高速化しているがXBox360上だと再生時間の1/6はかかる。CPUが良ければすぐに終わるが)
            //4.コンテンツパイプラインでビルドする場合、ファイルサイズが当然ながら大きくなる(XBoxでの読み込みは早いです)
            
            //model1のモデルのプレイヤーを呼び出し
            //モーションを0番トラックにセットする
            model1.Player.SetMotion(0, motion1, true);
            //0番モーションをベイクする(これで高速化される
            model1.Player.BakeMotion(0);
            
            //コンテンツパイプライン中でのベイク方法はXMLを使用する
            //作成方法
            //-まずはTrueMyHeartB.xmlに書かれているように記述する(この辺りはひにけにXNAさんが詳しいのでそちらを参照)
            //-次にコンテンツプロセッサを「MikuMikuDanceモーションベイク:MikuMikuDancd for XNA」にする
            //LoadBakedMotionで.xmlを読み込む
            
            //ベイク済みモーションの読み込み
            motion2 = mmd.LoadBakedMotion("TrueMyHeartB");
            //モーションを0番トラックにセットする
            model2.Player.SetMotion(0, motion2, true);
        }

        /// <summary>
        /// UnloadContent関数
        /// アンロードが必要な物はここでアンロードを行う
        /// </summary>
        protected override void UnloadContent()
        {
        }

        /// <summary>
        /// アップデート関数
        /// 毎フレーム処理する描画以外のコードをここに記述する
        /// </summary>
        /// <param name="gameTime">現在の時間のスナップショットを表すオブジェクト</param>
        protected override void Update(GameTime gameTime)
        {
            //XBoxコントローラーのBackボタンを押したら終了
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();
            //キーボードのEscキーを押したら終了
            if (Keyboard.GetState().IsKeyDown(Keys.Escape))
                this.Exit();
            //エンターキーorAボタンを押したとき
            if (Keyboard.GetState().IsKeyDown(Keys.Enter) || GamePad.GetState(PlayerIndex.One).Buttons.A == ButtonState.Pressed)
            {
                //現在再生中でなければ
                if (!model1.Player.IsPlay(0))
                {
                    //0番トラックにセットされているモーションをリセット
                    model1.Player.Reset(0);
                    model2.Player.Reset(0);
                    //0番トラックにセットされているモーションを再生する
                    model1.Player.Start(0, false);
                    model2.Player.Start(0, false);
                }
            }

            //モデルの再生処理はDrawableGameComponentを継承しているため
            //XNAにより自動で処理が行われる

            //XNA内部のUpdate処理を行う
            base.Update(gameTime);
        }

        /// <summary>
        /// Draw関数
        /// 毎フレーム処理する描画コードをここに記述する
        /// </summary>
        /// <param name="gameTime">現在の時間のスナップショットを表すオブジェクト</param>
        protected override void Draw(GameTime gameTime)
        {
            //画面をCornflowerBlue色で消去する
            GraphicsDevice.Clear(Color.CornflowerBlue);

            //モデルはDrawableGameComponentを継承しているため
            //XNAにより自動で描画される

            //XNA内部のDraw処理を行う
            base.Draw(gameTime);
        }
    }
}
